/*
	CVSNT Generic API
    Copyright (C) 2004 Tony Hoyle and March-Hare Software Ltd

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License version 2.1 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <config.h>
#include "lib/api_system.h"
#include "cvs_string.h"
#include "ServerIO.h"
#include "SqlConnection.h"
#include "LibraryAccess.h"

#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <winsock2.h>
#else
#include <sys/socket.h>
#include <netinet/in.h>
#endif

#ifdef _WIN32
#define STATIC_DB /* We can use late binding in Win32 to be more efficient */
#else
#undef STATIC_DB
#endif

#ifdef STATIC_DB

#ifdef HAVE_MYSQL
        #ifndef _WIN32
                #include <mysql.h>
        #else
                /* Win32 includes its own copy */
                #include "mysql/mysql-3.23/mysql.h"
        #endif
        #include "mysql/MySqlConnection.h"
#endif

#ifdef HAVE_SQLITE
        #include <sqlite3.h>
        #include "sqlite/SQLiteConnection.h"
#endif

#ifdef HAVE_POSTGRES
        #include <libpq-fe.h>
        #include "postgres/PostgresConnection.h"
#endif

#ifdef HAVE_ODBC
        #include <sql.h>
        #include "odbc/OdbcConnection.h"
#endif

#include "sqlite/SQLiteConnection.h"
#include "mysql/MySqlConnection.h"
#include "postgres/PostgresConnection.h"
#include "odbc/OdbcConnection.h"
#endif /* STATIC_DB */

CSqlConnection* CSqlConnection::Alloc(SqlConnectionType type, const char *dir)
{
#ifdef STATIC_DB
	switch(type)
	{
#ifdef HAVE_SQLITE
	case sqtSqlite:
		CServerIo::trace(3,"Connecting to SQLite");
		return new CSQLiteConnection();
#endif
#ifdef HAVE_MYSQL
	case sqtMysql:
		CServerIo::trace(3,"Connecting to MySql");
		return new CMySqlConnection();
#endif
#ifdef HAVE_POSTGRES
	case sqtPostgres:
		CServerIo::trace(3,"Connecting to Postgres");
		return new CPostgresConnection();
#endif
#ifdef HAVE_ODBC
	case sqtOdbc:
		CServerIo::trace(3,"Connecting to Odbc");
		return new COdbcConnection();
#endif
	default:
		return NULL;
	}
#else /* not _WIN32 */
	CLibraryAccess la;
	CSqlConnection* (*pNewSqlConnection)(unsigned ver) = NULL;

	switch(type)
	{
	case sqtSqlite:
		CServerIo::trace(3,"Connecting to SQLite");
		if(!la.Load("sqlite_database"SHARED_LIBRARY_EXTENSION,dir))
			return false;
		break;
	case sqtMysql:
		CServerIo::trace(3,"Connecting to MySql");
		if(!la.Load("mysql_database"SHARED_LIBRARY_EXTENSION,dir))
			return false;
		break;
	case sqtPostgres:
		CServerIo::trace(3,"Connecting to Postgres");
		if(!la.Load("postgres_database"SHARED_LIBRARY_EXTENSION,dir))
			return false;
		break;
	case sqtOdbc:
		CServerIo::trace(3,"Connecting to Odbc");
		if(!la.Load("odbc_database"SHARED_LIBRARY_EXTENSION,dir))
			return false;
		break;
	default:
		return NULL;
	}
	pNewSqlConnection = (CSqlConnection*(*)(unsigned ver))la.GetProc("NewSqlConnection");
	if(!pNewSqlConnection)
		return NULL;
	CSqlConnection *conn = pNewSqlConnection(0);
	la.Detach(); // Add a reference so the library never unloads
	return conn;
#endif
}

