/*
	CVSNT Lockserver - Publish over mdns
    Copyright (C) 2004 Tony Hoyle and March-Hare Software Ltd

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/* NB the above may be largely irrelevant - a lot of this is based on the example
   code which is BSD licensed...  feel free to treat this file as BSD if it helps */

/* This isn't very efficient at the moment, either... */

#include "config.h"
#ifdef ENABLE_ZEROCONF
#include "../cvsapi/cvsapi.h"
#include "LockService.h"
#include "mdns.h"
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <ctype.h>
#ifdef HAVE_SYS_PARAM_H
#include <sys/param.h>
#endif
#include <errno.h>
#include <cvstools.h>

extern bool g_bTestMode;

/* *WHY* didn't they defined a standard interface for this stuff? */

static const char *getHostname()
{
	static char host[1024];

	if(!CGlobalSettings::GetGlobalValue("cvsnt","PServer","ServerName",host,sizeof(host)))
		return host;

	if(gethostname(host,sizeof(host)))
		return "localhost?";
	char *p=strchr(host,'.');
	if(p) *p='\0';
	return host;
}

/* Howl - Unix/Win32. */
#if defined(HAVE_HOWL_H)

/* Howl includes a config.h
  from the platform it was compliled on. Bleh. */

#ifdef HAVE_HOWL_H
#undef HAVE_CONFIG_H
#include <howl.h>
#endif
#include <stdio.h>

static sw_salt g_salt;
static sw_discovery discovery;

static sw_result HOWL_API service_reply(sw_discovery discovery, sw_discovery_oid oid,
				sw_discovery_publish_status	status,	sw_opaque extra)
{
	static sw_string status_text[] =
	{
		"Zeroconf data published.",
		"Zeroconf server stopped.",
		"This server name is already in use.  Zeroconf data not published.",
		"Invalid Zeroconf data"
	};

	fprintf(stderr, "%s\n", status_text[status]);
	return SW_OKAY;
}

int mdns_publish(int port)
{
	sw_result						result;
	sw_discovery_publish_id	id;
	cvs::string txtRecord,rep,desc,tmp,servername;

	if(g_bTestMode)
		printf("Publishing mdns records..\n");

	if (sw_discovery_init_with_flags(&discovery,(sw_discovery_init_flags)(SW_DISCOVERY_USE_SHARED_SERVICE|SW_DISCOVERY_USE_PRIVATE_SERVICE)) != SW_OKAY)
	{
		fprintf(stderr, "sw_discovery_init() failed\n");
		return -1;
	}

	servername = getHostname();

	if ((result = sw_discovery_publish(discovery, 0, servername.c_str(), "_cvspserver._tcp.", NULL, NULL, port, NULL, 0, service_reply, NULL, &id)) != SW_OKAY)
	{
		fprintf(stderr, "publish failed: %d\n", result);
		return -1;
	}

	sw_discovery_salt(discovery,&g_salt);

	return 0;
}

int mdns_step()
{
	sw_ulong msecs = 500;
	sw_salt_step(g_salt,&msecs);
	return 0;
}

int mdns_close()
{
	sw_discovery_fina(discovery);
	return 0;
}

#elif defined (HAVE_DNS_SD_H) /* Apple version */
#include <dns_sd.h>

#ifndef DNSSD_API
#define DNSSD_API
#endif

static DNSServiceRef client  = NULL;
static DNSRecordRef record = NULL;
typedef union { unsigned char b[2]; unsigned short NotAnInteger; } Opaque16;

static void DNSSD_API reg_reply(DNSServiceRef client, DNSServiceFlags flags, DNSServiceErrorType errorCode,
	const char *name, const char *regtype, const char *domain, void *context)
{
	printf("Got a reply for %s.%s%s: ", name, regtype, domain);
	switch (errorCode)
		{
		case kDNSServiceErr_NoError:      printf("Name now registered and active\n"); break;
		case kDNSServiceErr_NameConflict: printf("Name in use, please choose another\n"); exit(-1);
		default:                          printf("Error %d\n", errorCode); return;
		}
}

int mdns_publish(int port)
{
	Opaque16 registerPort = { { port >> 8, port & 0xFF } };
	cvs::string rep,desc,tmp,servername;

	servername = getHostname();

	DNSServiceErrorType  err = DNSServiceRegister(&client, 0, 0, servername.c_str(), "_cvspserver._tcp.", "", NULL, registerPort.NotAnInteger, 0, NULL, reg_reply, NULL);
	if (!client || err != kDNSServiceErr_NoError) { printf("Unable to register with mDNS responder (%d)\n",err); return (-1); }
		return 0;
}

int mdns_step()
{
	if(!client)
		return 0;

	int dns_sd_fd  = client  ? DNSServiceRefSockFD(client ) : -1;
	int nfds = dns_sd_fd + 1;
	fd_set readfds;
	struct timeval tv;
	int result;

	// 1. Set up the fd_set as usual here.
	// This example client has no file descriptors of its own,
	// but a real application would call FD_SET to add them to the set here
	FD_ZERO(&readfds);

	// 2. Add the fd for our client(s) to the fd_set
	if (client ) FD_SET(dns_sd_fd , &readfds);

	// 3. Set up the timeout.
	tv.tv_sec = 1;
	tv.tv_usec = 0;

	result = select(nfds, &readfds, (fd_set*)NULL, (fd_set*)NULL, &tv);
	if (result > 0)
	{
		DNSServiceErrorType err = kDNSServiceErr_NoError;
		if      (client  && FD_ISSET(dns_sd_fd , &readfds)) err = DNSServiceProcessResult(client );
		if (err) { fprintf(stderr, "DNSServiceProcessResult returned %d", err); }
	}
	else if (result == 0)
	{
	}
	else
	{
		printf("select() returned %d errno %d %s\n", result, errno, strerror(errno));
	}
	return 0;
}

int mdns_close()
{
	if (client ) DNSServiceRefDeallocate(client );
	return 0;
}

#else

/* This code never gets used, except possibly for development */

int mdns_publish(int port)
{
	return 0;
}

int mdns_step()
{	
	sleep(10);
	return 0;
}

int mdns_close()
{
	return 0;
}

#endif
#endif /* ENABLE_ZEROCONF */
