#ifndef fooqueryhfoo
#define fooqueryhfoo

/* $Id: mdnsclient.h,v 1.1.2.5 2005/09/08 12:28:51 tmh Exp $ */

/***
  This file is part of nss-mdns.
 
  nss-mdns is free software; you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  nss-mdns is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.
 
  You should have received a copy of the GNU Lesser General Public
  License along with nss-mdns; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.
***/

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _WIN32
#include <stdarg.h> // hpux needs this
#include <inttypes.h>
#else
#include <winsock2.h>
#include <ws2tcpip.h>
#include "win32/inttypes.h"
#endif

#ifndef MDNS_EXPORT
#define MDNS_EXPORT
#endif

typedef struct ipv4_address_struct_t {
	uint8_t address[4];
} ipv4_address_t;

typedef struct ipv6_address_struct_t {
    uint8_t address[16];
} ipv6_address_t;

#ifdef _WIN32
typedef SOCKET sock_t;
#else
typedef int sock_t;
#endif

MDNS_EXPORT sock_t mdns_open_socket(void);
MDNS_EXPORT int mdns_close_socket(sock_t sock);

struct service_handlers
{
	void (*service_name_func)(const char *name, void *userdata);
	void (*service_srv_func)(const char *name, uint16_t priority, uint16_t weight, uint16_t port, const char *target, void *userdata);
	void (*service_txt_func)(const char *name, const char *txt, void *userdata);
	void (*service_ipv4_func)(const char *name, const ipv4_address_t *ipv4, void *userdata);
	void (*service_ipv6_func)(const char *name, const ipv6_address_t *ipv6, void *userdata);
};


MDNS_EXPORT int mdns_query_services(sock_t fd, 
				const char *prefix,
				struct service_handlers *service_func,
				void *userdata, uint64_t timeout);

MDNS_EXPORT int mdns_query_name(sock_t fd,
               const char *name,
               void (*ipv4_func)(const ipv4_address_t *ipv4, void *userdata),
               void (*ipv6_func)(const ipv6_address_t *ipv6, void *userdata),
               void *userdata);

#ifndef NSS_IPV6_ONLY
MDNS_EXPORT int mdns_query_ipv4(sock_t fd,
               const ipv4_address_t *ipv4,
               void (*name_func)(const char *name, void *userdata),
               void *userdata);
#endif
#ifndef NSS_IPV4_ONLY
MDNS_EXPORT int mdns_query_ipv6(sock_t fd,
               const ipv6_address_t *ipv6,
               void (*name_func)(const char *name, void *userdata),
               void *userdata);
#endif

#undef MDNS_EXPORT

#ifdef __cplusplus
}
#endif

#endif
